# Clover.tcl---RIS Corp Clover module control frame



set frame "Clover"
set frametitle "Clover Module Setup & Monitor"
set icontitle "Clover"

set mname "clover"

set maxmodule 32

set maxmodule [inform camaccontrol inquire [set mname]count]
set modulelist {}
for {set i 1} {$i <= $maxmodule} {incr i} {lappend modulelist $i}

set module [first $modulelist]

set w2dac(CFDTHRESHA) 1
set w2dac(CFDWIDTHA)  2
set w2dac(PILEUPTHRESHA) 3
set w2dac(CFDTHRESHB) 4
set w2dac(CFDWIDTHB)  5
set w2dac(PILEUPTHRESHB) 6
set w2dac(CFDTHRESHC) 7
set w2dac(CFDWIDTHC)  8
set w2dac(PILEUPTHRESHC) 9
set w2dac(CFDTHRESHD) 10
set w2dac(CFDWIDTHD)  11
set w2dac(PILEUPTHRESHD) 12
set w2dac(GE1) 13
set w2dac(GE2) 14
set w2dac(GE3) 15
set w2dac(BGO1) 16
set w2dac(BGO2) 17
set w2dac(BGO3) 18
set w2dac(BGO4) 19
set w2dac(BGO5) 20
set w2dac(BGO6) 21
set w2dac(BGO7) 22
set w2dac(BGO8) 23
set w2dac(BGOSUM) 24
set w2dac(CFDDELAYA) 25
set w2dac(CFDDELAYB) 26
set w2dac(CFDDELAYC) 27
set w2dac(CFDDELAYD) 28
set w2dac(GECLEAN) 29

set mpx1 0
set mpx2 0
set mpx3 0
set mpx4 0


loadsource common.camac

set reset_cmd Vsn


proc repaint {} {
  global module
  :modify MODULE-NUM :value $module
}

proc click-select-adc {v} {}

proc click-redisplay {} {
   redisplay-module
   redisplay-dacs
   redisplay-mpx
   redisplay-ctl
}
 
proc redisplay-module {} {
   camac-address
   :modify VSN  :value [expr [read-reg Vsn] & 255]
}

proc redisplay-dacs {} {
   global w2dac

   foreach w [array names w2dac] {
       set dac $w2dac($w)
       set vl [read-reg DAC$dac]
       if {($dac >= 1) && ($dac <=24)} {
	 set revalue3  [expr (($vl & 255) -127.5) * 0.023529412]
#	 puts $revalue3
       }
       if {($dac >= 25) && ($dac <= 29)} {
	   set revalue3 [expr ($vl & 255)/2.55 + 100 ]
#	   puts $revalue3
       }
       :modify $w :value $revalue3
   }
}

proc redisplay-mpx {} {

    set mpx [read-reg Mpx]
    set mpx3 [expr $mpx & 7]
    set mpx4 [expr ($mpx >> 4) & 7]
    set mpx1 [expr ($mpx >> 8) & 7]
    set mpx2 [expr ($mpx >> 12) & 7]

#  insert-debug "mpx register $mpx $mpx1 $mpx2 $mpx3 $mpx4"

    :modify MPX1 :value $mpx1
    :modify MPX2 :value $mpx2
    :modify MPX3 :value $mpx3
    :modify MPX4 :value $mpx4
}

proc redisplay-ctl {} {

    set ctl [read-reg Ctrl]
    set ctl1 [expr $ctl & 1]
    set ctl2 [expr ($ctl >> 1) & 1]
    set ctl3 [expr ($ctl >> 2) & 1]

    :modify CTL1 :value $ctl1
    :modify CTL2 :value $ctl2
    :modify CTL3 :value $ctl3
}

proc set_ctl {w v} {
#  insert-debug "set control bit $w to $v"

    set ctl [read-reg Ctrl]
    set ctl1 [expr $ctl & 1]
    set ctl2 [expr ($ctl >> 1) & 1]
    set ctl3 [expr ($ctl >> 2) & 1]

#  insert-debug "ctl register $ctl1 $ctl2 $ctl3"

    set ctl$w $v

    set ctl [expr (($ctl3 << 1) + $ctl2 << 1) + $ctl1]
    write-reg Ctrl $ctl

    redisplay-ctl
}

proc set_mpx {w v} {
#  insert-debug "set mpx $w to $v"

    set mpx [read-reg Mpx]
    set mpx3 [expr $mpx & 7]
    set mpx4 [expr ($mpx >> 4) & 7]
    set mpx1 [expr ($mpx >> 8) & 7]
    set mpx2 [expr ($mpx >> 12) & 7]

#   insert-debug "mpx register $mpx $mpx1 $mpx2 $mpx3 $mpx4"

    set mpx$w $v

    set mpx [expr (((((($mpx2) << 4) + $mpx1) << 4) + $mpx4 + 0x8) << 4) + $mpx3 + 0x8]
    write-reg Mpx $mpx

#  insert-debug "setting mpx register to $mpx"

    redisplay-mpx
}

proc set_dac {w v} {
   global w2dac
#  insert-debug "set dac widget $w to $v"
     set dac $w2dac($w)
     set i [expr ($dac-1)/8]
     set k [expr ($dac - (8*$i))]
     if {($dac >= 1) && ($dac <= 24)} {
#	 puts $v
	 set revalue [expr round(($v*255/6)+ 127.5) ]
#	 puts $revalue
     }
     if {($dac >= 25) && ($dac <= 29)} {
#	 puts $v
	 set revalue [expr round(($v - 100) * 2.55) ]
#	 puts $revalue
     }
     set vl [expr $revalue + ($k*256)]
#  insert-debug "setting register DAC$dac to $vl ($i $k $v)"
     write-reg DAC$dac $vl

#    redisplay this DAC

     set vl [read-reg DAC$dac]
      if {($dac >= 1) && ($dac <= 24)} {
#	 puts [expr $vl & 255]
	 set revalue2  [expr (($vl & 255) -127.5) * 0.023529412]
#	 puts $revalue2
     }
     if {($dac >= 25) && ($dac <= 29)} {
	 set revalue2 [expr ($vl & 255)/2.55 + 100 ]
#	 puts $revalue2
     }
     :modify $w :value $revalue2
}

proc do_initialise {args} {
   global w2dac

   foreach w [array names w2dac] {
       set dac $w2dac($w)
       write-reg DAC$dac 127
   }

   write-reg Mpx 0x000088
   
   write-reg Ctrl 0

   click-redisplay
}


proc click arg {

   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit; return}
   switch -glob $w {
      MODULE-SEL  -
      MODULE-NUM  {click-select-module $v; click-redisplay}
      ALL-MODULES {click-all-modules $v}

      CTL*	{set_ctl [string range $w 3 3] $v}

      MPX*      {set_mpx [string range $w 3 3] $v}

      default   {set_dac $w $v }
   }
}


proc make-frame {} {
  global env
  global maxmodule modulelist camcrate camstation

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/camac_module" \
      :icon-name "[self icon-title]" :resize F

  :panel MODULE  :layout T :padx 10 :pady 10
      :layout H; :row-gap 10

     :number MODULE-NUM  :label "Module number:" :min 1 :max $maxmodule :w 5
     :menubutton MODULE-SEL :label "select:" :strings $modulelist 
     :hskip 50
     :text CAM-CRATE   :label "CAMAC Crate:"   :w 5 :value $camcrate :deaf T
     :text CAM-STATION :label "CAMAC Station:" :w 5 :value $camstation :deaf T

     :next-row T
     :message M1 :label "Module Control:" :bold T
     :hskip 300
     :checkbox ALL-MODULES :label "Act on ALL modules?"
     :next-row T
     :col-gap 20

     :text VSN :label VSN: :w 5 :deaf T
     :checkbox CTL1  :label "FERA Write Mode"
     :checkbox CTL2  :label "Pile Up Mode"
     :checkbox CTL3  :label "Gain Mode"

   :container C1 :layout T 

   :panel SEGA :layout L
      :layout V; :row-gap 10
   
     :message GEA :label "Ge A High Resolution" :bold T
     :slider CFDTHRESHA     :label "CFD Threshold:"     :min -3 :max 3 :resolution 0.0234375 :valuex 16c
     :slider CFDWIDTHA      :label "CFD Width:"         :min -3 :max 3 :resolution 0.0234375 :valuex 16c
     :slider CFDDELAYA      :label "CFD Delay:"         :min 100 :max 200 :resolution 0.390625 :valuex 16c
     :slider PILEUPTHRESHA  :label "PileUp  Threshold:" :min -3 :max 3 :resolution 0.0234375 :valuex 16c

   :panel SEGB :layout L :pady 10
      :layout V; :row-gap 10
   
     :message GEB :label "Ge B High Resolution" :bold T
     :slider CFDTHRESHB     :label "CFD Threshold:"     :min -3 :max 3 :resolution 0.0234375 :valuex 16c
     :slider CFDWIDTHB      :label "CFD Width:"         :min -3 :max 3 :resolution 0.0234375 :valuex 16c
     :slider CFDDELAYB      :label "CFD Delay:"         :min 100 :max 200 :resolution 0.390625 :valuex 16c
     :slider PILEUPTHRESHB  :label "PileUp  Threshold:" :min -3 :max 3 :resolution 0.0234375 :valuex 16c

   :container C2 :layout T 

   :panel SEGC :layout L
      :layout V; :row-gap 10
   
     :message GEC :label "Ge C High Resolution" :bold T
     :slider CFDTHRESHC     :label "CFD Threshold:"     :min -3 :max 3 :resolution 0.0234375 :valuex 16c
     :slider CFDWIDTHC      :label "CFD Width:"         :min -3 :max 3 :resolution 0.0234375 :valuex 16c
     :slider CFDDELAYC      :label "CFD Delay:"         :min 100 :max 200 :resolution 0.390625 :valuex 16c
     :slider PILEUPTHRESHC  :label "PileUp  Threshold:" :min -3 :max 3 :resolution 0.0234375 :valuex 16c

   :panel SEGD :layout L :pady 10
      :layout V; :row-gap 10
   
     :message GED :label "Ge D High Resolution" :bold T
     :slider CFDTHRESHD     :label "CFD Threshold:"     :min -3 :max 3 :resolution 0.0234375 :valuex 16c
     :slider CFDWIDTHD      :label "CFD Width:"         :min -3 :max 3 :resolution 0.0234375 :valuex 16c
     :slider CFDDELAYD      :label "CFD Delay:"         :min 100 :max 200 :resolution 0.390625 :valuex 16c
     :slider PILEUPTHRESHD  :label "PileUp  Threshold:" :min -3 :max 3 :resolution 0.0234375 :valuex 16c

   :container end

   :panel GELO :layout T
     :layout H; :col-gap 10; :row-gap 10
     :message BGOM  :label "Ge Low Resolution Thresholds"  :bold T
     :next-row T

      :slider GE1  :label "1"  :min -3 :max 3 :resolution 0.0234375 
      :slider GE2  :label "2"  :min -3 :max 3 :resolution 0.0234375 
      :slider GE3  :label "3"  :min -3 :max 3 :resolution 0.0234375

   :panel P1 :layout T  :pady 10
     :layout H; :col-gap 10; :row-gap 10
     :vskip 10
     :slider GECLEAN :label "Ge Clean Delay"     :min 100 :max 200 :resolution 0.390625 
     :slider BGOSUM  :label "BGO Sum Threshold"  :min -3 :max 3 :resolution 0.0234375

   :panel BGO :layout T
     :layout H; :col-gap 10; :row-gap 10
     :message BGOM  :label "BGO Thresholds"  :bold T
     :next-row T

      :slider BGO1  :label "1"  :min -3 :max 3 :resolution 0.0234375
      :slider BGO2  :label "2"  :min -3 :max 3 :resolution 0.0234375 
      :slider BGO3  :label "3"  :min -3 :max 3 :resolution 0.0234375
      :next-row T

      :slider BGO4  :label "4"  :min -3 :max 3 :resolution 0.0234375 
      :slider BGO5  :label "5"  :min -3 :max 3 :resolution 0.0234375 
      :slider BGO6  :label "6"  :min -3 :max 3 :resolution 0.0234375
      :next-row T

      :slider BGO7  :label "7"  :min -3 :max 3 :resolution 0.0234375
      :slider BGO8  :label "8"  :min -3 :max 3 :resolution 0.0234375

   :panel MPX :layout T :pady 10
     :layout H; :col-gap 50; :row-gap 10
     :message MPXM  :label "Front Panel Multiplixer Selection"  :bold T
     :next-row T

     :hskip 20
     :choice-stack MPX1  :label "1" :menu [list CFD1 CFD2 CFD3 CFD4 SUMLOGIC GEDELAY BGOTEST PU_SUM]
     :choice-stack MPX2  :label "2" :menu [list BGOHIT1 BGOHIT2 BGOHIT3 BGOHIT4 BGOHIT5 BGOHIT6 BGOHIT7 BGOHIT8]
     :choice-stack MPX3  :label "3" :menu [list GeMON1 GeMON2 GeMON3 GeMON4 SMON1 SMON2 SMON3 BGOMON]
     :choice-stack MPX4  :label "4" :menu [list FAST1 FAST2 FAST3 FAST4 SMON1 SMON2 SMON3 BGOMON]

   :panel CLICK :layout B :padx 20 :after FOOTER
      :layout H; :col-gap 25
   
     :button REDISPLAY  :label "Redisplay"          :command click-redisplay
     :button INITIALISE :label "Initialise Module"  :command do_initialise

  :show T
}
